@@ -48,12 +48,13 @@ def load_program(program_input):
     with open(program_input + '/loss_name', 'r') as fin:
         loss_name = fin.read()
     if os.path.exists(program_input + '/acc'):
-        target = []
+        target = {}
         with open(program_input + '/acc', 'r') as fin:
             for line in fin:
+                cur_target = line.split(" ")[0]
                 for var in new_main.list_vars():
-                    if var.name == line[:-1]:
-                        target.append(var)
+                    if var.name == line.split(" ")[1].strip():
+                        target[cur_target] = var
     else:
         print("Please save your target first")
         target = None
@@ -147,7 +148,6 @@ def save_program(main_prog,
     main_program_str = main_prog.desc.serialize_to_string()
     startup_program_str = startup_prog.desc.serialize_to_string()
     params = main_prog.global_block().all_parameters()
-    para_info = []
     with open(program_path + '/input_names', 'w') as fout:
         for input in input_list:
             fout.write("%s\n" % input)
@@ -181,5 +181,6 @@ def save_program(main_prog,
         with open(program_path + '/checkpoint', 'w') as fout:
             for ckpt in checkpoints:
                 fout.write("%s\n" % ckpt.name)
-    with open(program_path + '/target', 'w') as fout:
-        fout.write(target.name)
+    with open(program_path + '/acc', 'w') as fout:
+        for t in target:
+            fout.write("{} {}\n".format(t, target[t]))
