@@ -17,33 +17,52 @@ from __future__ import division
 from __future__ import print_function
 import collections
 import io
+import random
+import paddle
 import six
 import os
 import gzip
 import paddle.fluid as fluid
 import numpy as np
+import paddle.distributed.fleet as fleet
 
 
 def load_bert_dataset(data_dir,
-                      vocab_path,
                       inputs,
                       batch_size,
+                      lang='ch',
+                      phase='train',
                       max_seq_len=512,
                       in_tokens=True,
+                      shuffle=True,
                       generate_neg_sample=True,
                       voc_size=21128):
     data_loader = fluid.io.DataLoader.from_generator(
         feed_list=inputs, capacity=70, iterable=True)
-    data_reader = DataReader(
-        data_dir=data_dir,
-        batch_size=batch_size,
-        in_tokens=in_tokens,
-        vocab_path=vocab_path,
-        voc_size=voc_size,
-        epoch=1,
-        max_seq_len=max_seq_len,
-        generate_neg_sample=generate_neg_sample)
+    if lang == 'en':
+        data_reader = DataReaderV2(
+            data_dir=data_dir,
+            phase=phase,
+            batch_size=batch_size,
+            in_tokens=in_tokens,
+            max_seq_len=max_seq_len,
+            shuffle_files=True)
+    elif lang == 'ch':
+        data_reader = DataReader(
+            data_dir=data_dir,
+            phase=phase,
+            batch_size=batch_size,
+            in_tokens=in_tokens,
+            vocab_path="{}/vocab.txt".format(data_dir),
+            voc_size=voc_size,
+            epoch=1,
+            max_seq_len=max_seq_len,
+            generate_neg_sample=generate_neg_sample)
 
+    else:
+        raise Exception(
+            "Bert Model only support chinese and english model now. Please check if your lang is one of 'ch' and 'en'"
+        )
     places = fluid.CUDAPlace(int(os.environ.get('FLAGS_selected_gpus', 0)))
     data_loader.set_batch_generator(data_reader.data_generator(), places)
     return data_loader
@@ -53,6 +72,7 @@ class DataReader(object):
     def __init__(self,
                  data_dir,
                  vocab_path,
+                 phase='train',
                  batch_size=4096,
                  in_tokens=True,
                  max_seq_len=512,
@@ -70,6 +90,7 @@ class DataReader(object):
         self.epoch = epoch
         self.current_epoch = 0
         self.current_file_index = 0
+        self.phase = phase
         self.total_file = 0
         self.current_file = None
         self.voc_size = voc_size
@@ -111,9 +132,8 @@ class DataReader(object):
         return [token_ids, sent_ids, pos_ids, label]
 
     def read_file(self, file):
-        assert file.endswith('.gz'), "[ERROR] %s is not a gzip file" % file
         file_path = self.data_dir + "/" + file
-        with gzip.open(file_path, "rb") as f:
+        with open(file_path, "rb") as f:
             for line in f:
                 parsed_line = self.parse_line(
                     line, max_seq_len=self.max_seq_len)
@@ -245,7 +265,10 @@ class DataReader(object):
         """
         data_generator
         """
-        files = os.listdir(self.data_dir)
+        if self.phase == 'train':
+            files = get_filelist(self.data_dir)
+        else:
+            files = get_val_filelist(self.data_dir)
         self.total_file = len(files)
         assert self.total_file > 0, "[Error] data_dir is empty"
 
@@ -304,6 +327,138 @@ class DataReader(object):
         return wrapper
 
 
+class DataReaderV2(object):
+    def __init__(self,
+                 data_dir,
+                 phase='train',
+                 batch_size=4096,
+                 in_tokens=True,
+                 max_seq_len=128,
+                 max_preds_per_seq=20,
+                 shuffle_files=True,
+                 is_test=False,
+                 epoch=100,
+                 **kwargs):
+        self.data_dir = data_dir
+        if phase == 'train':
+            file_list = get_filelist(data_dir)
+        else:
+            file_list = get_val_filelist(data_dir)
+        self.file_list = [data_dir + "/" + x for x in file_list]
+        self.batch_size = batch_size
+        self.max_seq_len = max_seq_len
+        self.epoch = epoch
+        self.shuffle = shuffle_files
+        self.in_tokens = in_tokens
+        self.is_test = is_test
+        self.current_epoch = -1
+        self.current_file_index = 0
+        self.total_file = 0
+        self.current_file = None
+
+    def convert_samples_to_fluid_tensors(self, samples):
+        src_ids = []
+        sent_ids = []
+        labels = []
+        input_mask = []
+        masked_lm_weight = []
+        masked_lm_ids = []
+        masked_lm_pos = []
+        batch_size = len(samples)
+        pos_ids = np.linspace(0, self.max_seq_len - 1, self.max_seq_len)
+        pos_ids = np.tile(pos_ids, (batch_size, 1))
+        def to_int_array(array_str):
+            array_list = array_str.split()
+            return [int(x) for x in array_list]
+
+        def to_float_array(array_str):
+            array_list = array_str.split()
+            return [float(x) for x in array_list]
+
+        for sample in samples:
+            sample = sample.strip()
+            group = sample.split(";")
+            src_ids.extend(to_int_array(group[0]))
+            input_mask.extend(to_int_array(group[1]))
+            sent_ids.extend(to_int_array(group[2]))
+            masked_lm_pos.extend(to_int_array(group[3]))
+            masked_lm_ids.extend(to_int_array(group[4]))
+            masked_lm_weight.extend(to_float_array(group[5]))
+            labels.extend(to_int_array(group[6]))
+
+        src_ids = np.array(src_ids).reshape(batch_size, self.max_seq_len)
+        sent_ids = np.array(sent_ids).reshape(batch_size, self.max_seq_len)
+        input_mask = \
+                np.array(input_mask).astype("int64").reshape(
+                    batch_size, self.max_seq_len)
+        labels = np.array(labels).astype("int64").reshape(batch_size, 1)
+        pos_ids = (input_mask * pos_ids).astype("int64")
+        input_mask = np.expand_dims(input_mask, axis=-1).astype("float32")
+
+        masked_lm_weight = np.array(masked_lm_weight).reshape(
+            batch_size, -1).astype("float32")
+        masked_lm_ids = np.array(masked_lm_ids).reshape(batch_size,
+                                                        -1).astype("int64")
+        masked_lm_pos = np.array(masked_lm_pos).reshape(batch_size,
+                                                        -1).astype("int64")
+
+        masked_cnt = masked_lm_weight.sum(axis=1).astype("int").flatten()
+        mask_label = np.empty((1, 0))
+        mask_pos = np.empty((1, 0))
+        for i, cnt in enumerate(masked_cnt):
+            mask_label = np.append(mask_label, masked_lm_ids[i, 0:cnt])
+            mask_pos = np.append(
+                mask_pos, masked_lm_pos[i, 0:cnt] + i * self.max_seq_len)
+
+        mask_label = mask_label.reshape([-1, 1]).astype("int64")
+        mask_pos = mask_pos.reshape([-1, 1]).astype("int64")
+
+        return src_ids, pos_ids, sent_ids, input_mask, mask_label, mask_pos, labels
+
+    def load_samples(self, filename, shuffle):
+        samples = []
+        with open(filename) as fin:
+            for line in fin:
+                samples.append(line)
+        if shuffle:
+            random.shuffle(samples)
+        return samples
+
+    def data_generator(self):
+        files = self.file_list
+        samples = []
+        batch_size = self.batch_size // self.max_seq_len if self.in_tokens \
+                     else self.batch_size
+        print("batch size: %d" % batch_size)
+        self.total_file = len(files)
+        self.current_file = "N/A"
+
+        def local_iter():
+            for epoch in range(self.epoch):
+                self.current_epoch = epoch
+                self.current_file_index = -1
+                for index, f in enumerate(files):
+                    self.current_file_index = index + 1
+                    self.current_file = f
+                    samples = self.load_samples(f, self.shuffle)
+
+                    def reader():
+                        for sample in samples:
+                            yield sample
+
+                    batch_iter = paddle.batch(reader, batch_size)
+                    for batch in batch_iter():
+                        converted_samples = self.convert_samples_to_fluid_tensors(
+                            batch)
+                        # print(converted_samples[0].shape)
+                        yield converted_samples
+
+        return local_iter
+
+    def get_progress(self):
+        return self.current_epoch, self.current_file_index, self.total_file, self.current_file
+
+
 def prepare_batch_data(insts,
                        total_token_num,
                        voc_size=0,
@@ -466,3 +621,46 @@ def mask(batch_tokens, total_token_num, vocab_size, CLS=1, SEP=2, MASK=3):
     mask_label = np.array(mask_label).astype("int64").reshape([-1, 1])
     mask_pos = np.array(mask_pos).astype("int64").reshape([-1, 1])
     return batch_tokens, mask_label, mask_pos
+
+
+def get_filelist(datadir):
+    if not os.path.exists("{}/train.txt".format(datadir)):
+        raise Exception("ERROR: Your data dir should include train.txt")
+
+    total_list = []
+    full_list = []
+    with open("{}/train.txt".format(datadir), 'r') as fin:
+        for line in fin:
+            current_file = line.strip()
+            if os.path.exists("{}/{}".format(datadir, current_file)):
+                total_list.append(current_file)
+            full_list.append(current_file)
+    total_num = len(total_list)
+    if len(total_list) == len(full_list):
+        print("files to train on this card: {}".format(total_list[
+            fleet.worker_index()::fleet.worker_num()]))
+        return total_list[fleet.worker_index()::fleet.worker_num()]
+    PADDLE_TRAINER_ENDPOINTS = os.environ.get('PADDLE_TRAINER_ENDPOINTS')
+    current_endpoint = os.environ.get('PADDLE_CURRENT_ENDPOINT')
+    endpoints = PADDLE_TRAINER_ENDPOINTS.split(",")
+    hostname, _ = current_endpoint.split(":")
+    host_endpoints = [x for x in endpoints if x.split(":")[0] == hostname]
+    current_id = host_endpoints.index(current_endpoint)
+    total_local_cards = len(host_endpoints)
+    print("files to train on this card: {}".format(total_list[
+        current_id::total_local_cards]))
+    return total_list[current_id::total_local_cards]
+
+
+def get_val_filelist(datadir):
+    if not os.path.exists("{}/val.txt".format(datadir)):
+        raise Exception("ERROR: Your data dir should include val.txt")
+
+    total_list = []
+    with open("{}/val.txt".format(datadir), 'r') as fin:
+        for line in fin:
+            current_file = line.strip()
+            if os.path.exists("{}/{}".format(datadir, current_file)):
+                total_list.append(current_file)
+
+    return total_list
