@@ -16,7 +16,10 @@ import argparse
 
 
 def parse_train_configs():
-    parser = argparse.ArgumentParser("fleet-lightning")
+    """
+    Set training configs using parse commends.
+    """
+    parser = argparse.ArgumentParser("fleet-x")
     parser.add_argument(
         "--lr", type=float, default=0.1, help="base learning rate")
     parser.add_argument(
@@ -31,10 +34,17 @@ def parse_train_configs():
         type=float,
         default=0.997,
         help="beta2 value for adam optimizer")
+    parser.add_argument(
+        "--batch_size", type=int, default=32, help="train batch size")
     parser.add_argument(
         "--epsilon",
         type=float,
         default=1e-9,
         help="epsilon value for adam optimizer")
+    parser.add_argument(
+        "--download_config",
+        type=str,
+        default=None,
+        help="yaml file used for data download")
     args = parser.parse_args()
     return args
